#include "DRV2605.h"
#include "i2c.h"
#define __PROG_TYPES_COMPAT__
#include <avr/pgmspace.h>

#if defined(ARDUINO) && ARDUINO >= 100
#include "Arduino.h"
#else
#include "WProgram.h"
#endif

#include "defs.h"
#include "DRV2605_defs.h"

DRV2605::DRV2605()
{
	TWCR |=(1 <<TWEN );	// Enable TWI
}

void DRV2605::init()
{
	return;
	// First time init
	I2C_Write( ADDR_MODE,			MODE_ACTIVE );	// Exit standby
	I2C_Write( ADDR_RTP_INPUT,		0x00 );
	I2C_Write( ADDR_WAV_SEQ1,		0x01 );
	I2C_Write( ADDR_WAV_SEQ2,		0x00 );
	I2C_Write( ADDR_ODT,			0x00 );
	I2C_Write( ADDR_SPT,			0x00 );
	I2C_Write( ADDR_SNT,			0x00 );
	I2C_Write( ADDR_BRT,			0x00 );
	I2C_Write( ADDR_A2H_MAX_IN,		0x64 );
}

bool DRV2605::autoCal( uint8_t ratedVoltage, uint8_t overdriveClamp, boolean LRA, uint8_t* compensation, uint8_t* backEMF, uint8_t* feedback )
{
	// Set defaults
	setDefaults();

	// Work out control registers
	uint8_t fb = FB_BRAKE_4x | FB_LOOP_FAST;// | (LRA ? FB_MODE_LRA : FB_MODE_ERM);
	uint8_t control1 = DEFAULT_CTRL1;

	if( LRA )
	{
		fb |= FB_MODE_LRA;
	}
	else
	{
		fb |= FB_MODE_ERM;
	}

	// Write required registers
	I2C_Write( ADDR_RATED_VOLT,		ratedVoltage );
	I2C_Write( ADDR_OD_CLAMP,		overdriveClamp );
	I2C_Write( ADDR_FEEDBACK,		fb );
	I2C_Write( ADDR_CTRL1,			control1 );
	I2C_Write( ADDR_MODE,			MODE_AUTOCAL );

	fb = GO;
	control1 = 0;
	I2C_Write( ADDR_GO,			GO );

	do
	{
		I2C_Read( ADDR_GO, &fb );
		delay( 100 );
		++control1;
	} while( fb & GO && control1 < 100 );

	// Read status bit
	I2C_Read( ADDR_STATUS, &fb );

	if( fb & STAT_DIAG_BAD )
	{
		//Results did not converge
		Serial.print(F( "Fail: " ));
		Serial.print( control1 );
		Serial.print(F( " tries. Status " ));
		Serial.println( fb, BIN );
		return false;
	}

	Serial.print(F( "Status " ));
	Serial.println( fb, BIN );

	I2C_Read( ADDR_AC_COMP,			compensation );
	I2C_Read( ADDR_AC_BACK_EMF,		backEMF );
	I2C_Read( ADDR_FEEDBACK,		feedback );
	return true;
}

void DRV2605::playFullHaptic( uint8_t library, uint8_t effect, uint8_t ratedVoltage, uint8_t overdriveClamp, uint8_t compensation, uint8_t backEMF, uint8_t feedback )
{
	// Set defaults
	setDefaults();

	// Work out control registers
	uint8_t fb = FB_BRAKE_4x | FB_LOOP_FAST | (6 == library ? FB_MODE_LRA : FB_MODE_ERM);	// Library 6 means LRA
	uint8_t control1 = DEFAULT_CTRL1;
	uint8_t control2 = DEFAULT_CTRL2;
	uint8_t control3 = DEFAULT_CTRL3;

	// Set saved BEMF Gain
	fb |= ( feedback & FB_BEMF_BITMASK );

	// Set open or closed loop based on library
	if( 1 == library )
		control3 |= CTRL3_ERM_OPEN;	// Set bit 5 for open loop operation
	else
		control3 &= ~CTRL3_ERM_OPEN;	// Clear bit 5 for closed loop operation

	// Write required registers
	I2C_Write( ADDR_RATED_VOLT,		ratedVoltage );
	I2C_Write( ADDR_OD_CLAMP,		overdriveClamp );
	I2C_Write( ADDR_FEEDBACK,		fb );
	I2C_Write( ADDR_AC_COMP,		compensation );
	I2C_Write( ADDR_AC_BACK_EMF,	backEMF );
	I2C_Write( ADDR_CTRL1,			control1 );
	I2C_Write( ADDR_CTRL2,			control2 );
	I2C_Write( ADDR_CTRL3,			control3 );
	I2C_Write( ADDR_LIBRARY,		library );
	I2C_Write( ADDR_WAV_SEQ1,		effect );
	I2C_Write( ADDR_WAV_SEQ2,		0x00 );	// Stop bit
	I2C_Write( ADDR_MODE,			MODE_ACTIVE );

	fb = GO;
	control1 = 0;
	I2C_Write( ADDR_GO,			GO );
	do
	{
		// For effect 118, we have to stop by clearing the GO bit.
		if ( 118 == effect ) 
		{
			// Let it run for 1 seconds.
			if ( 10 == control1 ) 
			{
				I2C_Write( ADDR_GO, STOP );     // Clear GO bit to stop effect.
			}

			delay( 100 );
			++control1;
		}

		// Read GO bit to see if effect is still playing.
		I2C_Read( 0x0C, &fb );   
	} while ( fb & GO );
}

void DRV2605::setDefaults()
{
	I2C_Write( ADDR_MODE,			DEFAULT_MODE );
	I2C_Write( ADDR_RTP_INPUT,		DEFAULT_RTP_INPUT );
	I2C_Write( ADDR_LIBRARY,		DEFAULT_LIBRARY );
	I2C_Write( ADDR_WAV_SEQ1,		DEFAULT_WAV_SEQ );
	I2C_Write( ADDR_WAV_SEQ2,		DEFAULT_WAV_SEQ );
	I2C_Write( ADDR_WAV_SEQ3,		DEFAULT_WAV_SEQ );
	I2C_Write( ADDR_WAV_SEQ4,		DEFAULT_WAV_SEQ );
	I2C_Write( ADDR_WAV_SEQ5,		DEFAULT_WAV_SEQ );
	I2C_Write( ADDR_WAV_SEQ6,		DEFAULT_WAV_SEQ );
	I2C_Write( ADDR_WAV_SEQ7,		DEFAULT_WAV_SEQ );
	I2C_Write( ADDR_WAV_SEQ8,		DEFAULT_WAV_SEQ );
	I2C_Write( ADDR_GO,				DEFAULT_GO );
	I2C_Write( ADDR_ODT,			DEFAULT_OD_OFFSET );
	I2C_Write( ADDR_SPT,			DEFAULT_ST_OFFSETP );
	I2C_Write( ADDR_SNT,			DEFAULT_ST_OFFSETN );
	I2C_Write( ADDR_BRT,			DEFAULT_BT_OFFSET );
	I2C_Write( ADDR_A2H_CTRL,		DEFAULT_A2H_CTRL );
	I2C_Write( ADDR_A2H_MIN_IN,		DEFAULT_A2H_MIN_IN );
	I2C_Write( ADDR_A2H_MAX_IN,		DEFAULT_A2H_MAX_IN );
	I2C_Write( ADDR_A2H_MIN_OUT,	DEFAULT_A2H_MIN_OUT );
	I2C_Write( ADDR_A2H_MAX_OUT,	DEFAULT_A2H_MAX_OUT );
	I2C_Write( ADDR_RATED_VOLT,		DEFAULT_RATED_VOLT );
	I2C_Write( ADDR_OD_CLAMP,		DEFAULT_OD_CLAMP );
	I2C_Write( ADDR_AC_COMP,		DEFAULT_AC_COMP );
	I2C_Write( ADDR_AC_BACK_EMF,	DEFAULT_AC_BACK_EMF );
	I2C_Write( ADDR_FEEDBACK,		DEFAULT_FEEDBACK );
	I2C_Write( ADDR_CTRL1,			DEFAULT_CTRL1 );
	I2C_Write( ADDR_CTRL2,			DEFAULT_CTRL2 );
	I2C_Write( ADDR_CTRL3,			DEFAULT_CTRL3 );
	I2C_Write( ADDR_AC_MEM,			DEFAULT_AC_MEM );
	I2C_Write( ADDR_VBAT_VOLT,		DEFAULT_VBAT_VOLT );
	I2C_Write( ADDR_LRA_RES,		DEFAULT_LRA_RES );
}

void DRV2605::I2C_Read( uint8_t address, uint8_t *data )
{
	noInterrupts();		// Protect I2C transaction from MPR121 interrupt.
	i2cSendStart();		// start
		i2cSendByte(DRV2605_ADDR_WR);	// write 0xB4
		i2cSendByte(address);			// write register address
		i2cSendStart();					// repeated start
		i2cSendByte(DRV2605_ADDR_RD);	// write 0xB5
		data[0] = i2cReceiveByte(0);	// NACK	
	i2cSendStop();
	TWCR &=~(1<< TWEN);	// Disable TWI
	TWCR |= (1<< TWEN);	// Enable TWI
	interrupts();

}

void DRV2605::I2C_Write( uint8_t address, uint8_t data )
{
	noInterrupts();		// Protect I2C transaction from MPR121 interrupt.
	i2cSendStart();
		i2cSendByte(DRV2605_ADDR_WR);	// write 0xB4
		i2cSendByte(address);			// write register address
		i2cSendByte(data);
	i2cSendStop();
	interrupts();
 
}
