// This Software demo for the Precision Microdrives Haptic Shield is provided
// under the MIT License

// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// deal in the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:

// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.

// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
// IN THE SOFTWARE.

/*
*     Update for REV 2.
*     now we control 9 bars instead 3button
*     so there is no WAIT FOR SELECT
*     Instead WAIT_FOR_LEFT we have swipe from left to right
*     instead WAIT_FOR_RIGHT we have swipe from right to left
*     when swipe bars on bottom are blanked in relation to position of bar which was touched
*
*
*
*
*/

// Include system headers
#define __PROG_TYPES_COMPAT__
#include <avr/pgmspace.h>
#include <EEPROM.h>
#include <SPI.h>
#include <Wire.h>

// Include our headers
//#include "types.h"
#include "defs.h"
#include "Metro.h"
#include "DRV2605.h"
#include "SSD1306LessRAM.h"

// Libraries
#include "mpr121.h"
#include "i2c.h"

// Objects
DRV2605 drv2605 = DRV2605();
Metro timer = Metro( 100 );
SSD1306	oled = SSD1306( OLED_MOSI, OLED_CLK, OLED_DC, OLED_RESET, OLED_CS );


static volatile uint16_t touchstatus, old_touchstatus, keylock;
static volatile long keypresstime;

static uint16_t lock_keys = 0;      // store code of which key is pressed and not released
static uint8_t keys_state_change = 0, current_key_motor;
static volatile uint16_t slider_values, slider;

// State of the test
enum State
{
  NONE,               //0
  STARTING,           //1
  WAITING_WIRE,       //2
  AUTOCAL_RUN,        //3
  AUTOCAL_SUCCESS,    //4
  AUTOCAL_FAIL,       //5
  WAITING_BACK,       //6
  BUTTON_BACK,        //7
  WAITING_PLAY,       //8
  BUTTON_PLAY,        //9
  WAITING_PMD,        //10
  BUTTON_PMD,         //11
  WAITING_SLIDE,      //12
  BUTTON_SLIDE,       //13

  FINISHED,           //14

} ;

State state = STARTING;

// Input
#define NUM_KEYS	5


State keyStates[ NUM_KEYS ] = {
  WAITING_BACK,
  WAITING_PLAY,
  WAITING_PMD,
  WAITING_SLIDE,
  NONE

  //	WAITING_LEFT,
  //	WAITING_SELECT,
  //	WAITING_RIGHT,
};

State nextStates[ NUM_KEYS ] = {

  NONE,
  BUTTON_BACK,
  BUTTON_PLAY,
  BUTTON_PMD,
  BUTTON_SLIDE
};

uint8_t keyPressEffects[ NUM_KEYS ] = {
  27, // Back effect -> Strong Double Click 1
  12, // Play effect -> Triple Click
  13,	// PMD effect -> Buzz 1
  1,  // Slide effect -> Strong Single Click 1
  //	7,	// Select effect -> Bump 1
  //	52,	// Right effect -> Pulse 1
};

// Motor
typedef struct {
  boolean LRA;			// TRUE if LRA, else ERM.
  uint8_t min_duty;		// Minimum PWM duty cycle to start ERM 0 = 0%, 511 = 100%
  uint8_t rated_duty;	// Rated PWM duty cycle to run ERM 0 = 0%, 511 = 100% = 3.3V
  uint8_t max_duty;		// Maximum PWM duty cycle to run ERM 0 = 0%, 511 = 100%
  char part_num[8];		// Part number string.
} MotorInfo;

MotorInfo motorInfo[]= {{ false,  56, 180, 250, "304-103" },   // M0 - ERM 304-106, min 1.3V, rated 3.1V, max 3.7V  
                       { true,  39,  110,  158, "C10-100" }};	// M1 - Leaded, LRA, C13-000 min 0.50 Vrms rated 2.30 Vrms max 3.30 Vrms

uint8_t compensation, backEMF, feedback;
uint8_t compensation1, backEMF1, feedback1;

// Wire tests
#define NUM_DEVICES	2//3
//byte addresses[ NUM_DEVICES ] = { 0x58, 0x5A, 0x5D };
byte addresses[ NUM_DEVICES ] = { 0x5A, 0x5D };

// Strings
//																	= "1234567890132456789012"
// - Checking Wire
static const char MSG_CHECK_WIRE_1[] PROGMEM = "Checking I2C";
// - Wire failed
static const char MSG_WIRE_FAILED_1[] PROGMEM = "I2C check failed";
static const char MSG_WIRE_FAILED_2[] PROGMEM = "Touch failed: %#.2X";
static const char MSG_WIRE_FAILED_3[] PROGMEM = "DRV failed:   %#.2X";
const char * const DEVICE_MSG[ NUM_DEVICES ] PROGMEM = {
  //	&MSG_WIRE_FAILED_3[ 0 ],	// 0x58: DRV
  &MSG_WIRE_FAILED_3[ 0 ],	// 0x5A: DRV
  &MSG_WIRE_FAILED_2[ 0 ] 	// 0x5D: Touch
};
// - Running AutoCal
static const char MSG_RUNNING_AUTOCAL_1[] PROGMEM = "Running AutoCal";
static const char MSG_RUNNING_AUTOCAL_2[] PROGMEM = "Shield should give";
static const char MSG_RUNNING_AUTOCAL_3[] PROGMEM = "1 second of feedback";
// - AutoCal succeeded
static const char MSG_AUTOCAL_SUCCESS_1[] PROGMEM = "AutoCal passed.";
static const char MSG_AUTOCAL_SUCCESS_2[] PROGMEM = "Results are:";
static const char MSG_AUTOCAL_SUCCESS_3[] PROGMEM = "LRA %#.2x %#.2x %#.2x";
static const char MSG_AUTOCAL_SUCCESS_4[] PROGMEM = "ERM %#.2x %#.2x %#.2x";
static const char MSG_AUTOCAL_SUCCESS_5[] PROGMEM = "Press back button";
// - AutoCal failed
static const char MSG_AUTOCAL_FAILED_1[] PROGMEM = "AutoCal failed.";
static const char MSG_AUTOCAL_FAILED_2[] PROGMEM = "Check serial output";
// - Back button success
static const char MSG_BUTTON_BACK_SUCCESS_1[] PROGMEM = "Back button pressed";
static const char MSG_BUTTON_BACK_SUCCESS_2[] PROGMEM = "Press play button";
// - Play button success
static const char MSG_BUTTON_PLAY_SUCCESS_1[] PROGMEM = "Play button pressed";
static const char MSG_BUTTON_PLAY_SUCCESS_2[] PROGMEM = "Press PMD button";
// - PMD button success
static const char MSG_BUTTON_PMD_SUCCESS_1[] PROGMEM = "PMD button pressed";
static const char MSG_BUTTON_PMD_SUCCESS_2[] PROGMEM = "Slide blank bottom";
static const char MSG_BUTTON_PMD_SUCCESS_3[] PROGMEM = "Third line";
static const char MSG_BUTTON_PMD_SUCCESS_4[] PROGMEM = "Fourth line";
// - Slide success
static const char MSG_BUTTON_RIGHT_SUCCESS_1[] PROGMEM = "Test finished. Shield";
static const char MSG_BUTTON_RIGHT_SUCCESS_2[] PROGMEM = "should be playing";
static const char MSG_BUTTON_RIGHT_SUCCESS_3[] PROGMEM = "effect simoultaneous";
static const char MSG_BUTTON_RIGHT_SUCCESS_4[] PROGMEM = "on LRA and ERM";

char lineBuf[ 22 ];

void setup()
{
  // Setup serial
  Serial.begin( 9600 );
  Serial.print( "FreeMem=" );
  Serial.println( freeRAM() );

  // Begine Wire to test that all devices are connected correctly
  Wire.begin();

  // Setup pins
  pinMode( LRA_SEL,	OUTPUT );	// Output for LRA  (0 = connected)
  pinMode( ERM_SEL,	OUTPUT );	// Output for ERM  (0 = connected)
  pinMode( GRIP_SEL,	OUTPUT );	// Output for GRIP (0 = connected)
  pinMode( SRC_SEL,	OUTPUT );	// Output for switching DRV(1) or MOSFET(0)
  pinMode( DRV_2605_EN,	OUTPUT );	// Output for ERV2605 ENable
  pinMode( PWM_OUT,	OUTPUT );	// Output for PWM

  digitalWrite( LRA_SEL,	        LOW );	        // SelectLRA
  digitalWrite( ERM_SEL,	        HIGH );	        // ERM disconnected
  digitalWrite( GRIP_SEL,	        HIGH );	        // GRIP disconnected
  digitalWrite( SRC_SEL,	        HIGH );	        // DRV connected
  digitalWrite( DRV_2605_EN,	HIGH );	        // Disable the DRV2605 (low power mode)
  digitalWrite( PWM_OUT,		LOW );	        // PWM output low for now

  pinMode( GR_SEL1,	OUTPUT );	// Motor select 1
  pinMode( GR_SEL2,	OUTPUT );	// Motor select 2
  digitalWrite( GR_SEL1,	LOW );	        // select motor 0
  digitalWrite( GR_SEL2,	LOW );	        //

  setupTouchPad();
  drv2605.init();
  oled.ssd1306_init( SSD1306_SWITCHCAPVCC );
  oled.clear_display();
}

void loop()
{
  byte a, b, b1, c;


  keys_service();


  switch ( state )
  {
    case NONE:
      break;

    case STARTING:
      Serial.println( "Doing STARTING state" );
      oled.clear_display();
      sprintf_P( lineBuf, MSG_CHECK_WIRE_1 );
      oled.clear_buf();
      oled.drawstring( 0, 2, lineBuf, WHITE );

      state = WAITING_WIRE;
      break;

    case WAITING_WIRE:
      Serial.println( "Doing WAITING_WIRE state" );

      // Start with the failure state on the screen.
      // This will be overwritten is we pass the test
      oled.clear_display();
      sprintf_P( lineBuf, MSG_WIRE_FAILED_1 );
      oled.clear_buf();
      oled.drawstring( 0, 2, lineBuf, WHITE );

      // Code based off the i2c_scanner sample.
      // Uses Wire.endTransmission to see if a
      // device is connected to an address.
      a = c = 0;
      for ( b = 0 ; b < NUM_DEVICES ; ++b )
      {
        Wire.beginTransmission( addresses[ b ] );

        if ( 0 != Wire.endTransmission() )
        {
          // Failed to reach a device
          a = 1;
          const char* str = (char*)pgm_read_word( &(DEVICE_MSG[ b ]) );
          sprintf_P( lineBuf, str, addresses[ b ] );

          oled.clear_buf();
          oled.drawstring( 0, 3 + c++, lineBuf, WHITE );
        }
      }

      Serial.print( "I2C device check " );
      if ( 0 != a )
      {
        Serial.println( "failed" );
        state = NONE;
      }
      else
      {
        Serial.println( "passed" );
        sprintf_P( lineBuf, MSG_RUNNING_AUTOCAL_1 );
        oled.clear_buf();
        oled.drawstring( 0, 2, lineBuf, WHITE );
        sprintf_P( lineBuf, MSG_RUNNING_AUTOCAL_2 );
        oled.clear_buf();
        oled.drawstring( 0, 3, lineBuf, WHITE );
        sprintf_P( lineBuf, MSG_RUNNING_AUTOCAL_3 );
        oled.clear_buf();
        oled.drawstring( 0, 4, lineBuf, WHITE );

        state = AUTOCAL_RUN;
      }

      break;

    case AUTOCAL_RUN:
      Serial.println( "Doing AUTOCAL_RUN state" );
      compensation = backEMF = feedback = 0;
      compensation1 = backEMF1 = feedback1 = 0;
      
      b = drv2605.autoCal( motorInfo[1].rated_duty, motorInfo[1].max_duty, motorInfo[1].LRA, &compensation, &backEMF, &feedback );
        digitalWrite( LRA_SEL,          HIGH );          // LRA disconnected
        digitalWrite( ERM_SEL,          LOW );         // ERM connected
      
      b1 = drv2605.autoCal( motorInfo[0].rated_duty, motorInfo[0].max_duty, motorInfo[0].LRA, &compensation1, &backEMF1, &feedback1 );

        digitalWrite( LRA_SEL,          LOW );          // SelectLRA
        digitalWrite( ERM_SEL,          HIGH );         // ERM disconnected
      
      if ( b1 )
      {
        state = AUTOCAL_SUCCESS;
      }
      else
      {
        state = AUTOCAL_FAIL;
      }
      break;

    case AUTOCAL_SUCCESS:
      Serial.println( "Doing AUTOCAL_SUCCESS state good" );
      oled.clear_display();
      sprintf_P( lineBuf, MSG_AUTOCAL_SUCCESS_1 );
      oled.clear_buf();
      oled.drawstring( 0, 2, lineBuf, WHITE );
      sprintf_P( lineBuf, MSG_AUTOCAL_SUCCESS_2 );
      oled.clear_buf();
      oled.drawstring( 0, 3, lineBuf, WHITE );
      sprintf_P( lineBuf, MSG_AUTOCAL_SUCCESS_3, compensation, backEMF, feedback );
      oled.clear_buf();
      oled.drawstring( 0, 4, lineBuf, WHITE );
      sprintf_P( lineBuf, MSG_AUTOCAL_SUCCESS_4, compensation1, backEMF1, feedback1 );
       oled.clear_buf();
      oled.drawstring( 0, 5, lineBuf, WHITE );
      sprintf_P( lineBuf, MSG_AUTOCAL_SUCCESS_5 );
      oled.clear_buf();
      oled.drawstring( 0, 6, lineBuf, WHITE );
 
      state = WAITING_BACK;
      break;

    case AUTOCAL_FAIL:
      Serial.println( "Doing AUTOCAL_FAIL state bad" );
      oled.clear_display();
      sprintf_P( lineBuf, MSG_AUTOCAL_FAILED_1 );
      oled.clear_buf();
      oled.drawstring( 0, 2, lineBuf, WHITE );
      sprintf_P( lineBuf, MSG_AUTOCAL_FAILED_2 );
      oled.clear_buf();
      oled.drawstring( 0, 3, lineBuf, WHITE );

      state = NONE;
      break;

    case BUTTON_BACK:
      Serial.println( "Doing BUTTON_BACK state" );
      oled.clear_display();
      sprintf_P( lineBuf, MSG_BUTTON_BACK_SUCCESS_1 );
      oled.clear_buf();
      oled.drawstring( 0, 2, lineBuf, WHITE );
      sprintf_P( lineBuf, MSG_BUTTON_BACK_SUCCESS_2 );
      oled.clear_buf();
      oled.drawstring( 0, 3, lineBuf, WHITE );

      state = WAITING_PLAY;
      break;

    case BUTTON_PLAY:
      Serial.println( "Doing BUTTON_PLAY state" );
      oled.clear_display();
      sprintf_P( lineBuf, MSG_BUTTON_PLAY_SUCCESS_1 );
      oled.clear_buf();
      oled.drawstring( 0, 2, lineBuf, WHITE );
      sprintf_P( lineBuf, MSG_BUTTON_PLAY_SUCCESS_2 );
      oled.clear_buf();
      oled.drawstring( 0, 3, lineBuf, WHITE );

      state = WAITING_PMD;
      break;

    case BUTTON_PMD:
      Serial.println( "Doing BUTTON_PMD state" );
      oled.clear_display();
      sprintf_P( lineBuf, MSG_BUTTON_PMD_SUCCESS_1 );
      oled.clear_buf();
      oled.drawstring( 0, 2, lineBuf, WHITE );
      sprintf_P( lineBuf, MSG_BUTTON_PMD_SUCCESS_2 );
      oled.clear_buf();
      oled.drawstring( 0, 3, lineBuf, WHITE );
    

      state = WAITING_SLIDE;
      break;

  case WAITING_SLIDE:

      lineBuf[ 0 ] = ' ';
      lineBuf[ 1 ] = ' ';
      lineBuf[ 2 ] = ' ';
      lineBuf[ 3 ] = ' ';
      lineBuf[ 4 ] = ' ';
      slider = slider_values;
    
      for(int i=13 ; i>4; i--){
         if (slider & 0x1) lineBuf[i] = '*';
         else lineBuf[i] = '_';
         slider >>= 1;
      }
      
       lineBuf[ 14 ] = '\0';
      oled.clear_buf();
      oled.drawstring( 0, 7, lineBuf, WHITE ); 
      break;
      

    case BUTTON_SLIDE:
      Serial.println( "Doing BUTTON SLIDE state" );
      oled.clear_display();
      sprintf_P( lineBuf, MSG_BUTTON_RIGHT_SUCCESS_1 );
      oled.clear_buf();
      oled.drawstring( 0, 2, lineBuf, WHITE );
      sprintf_P( lineBuf, MSG_BUTTON_RIGHT_SUCCESS_2 );
      oled.clear_buf();
      oled.drawstring( 0, 3, lineBuf, WHITE );
      sprintf_P( lineBuf, MSG_BUTTON_RIGHT_SUCCESS_3 );
      oled.clear_buf();
      oled.drawstring( 0, 4, lineBuf, WHITE );
      sprintf_P( lineBuf, MSG_BUTTON_RIGHT_SUCCESS_4 );
      oled.clear_buf();
      oled.drawstring( 0, 5, lineBuf, WHITE );

      state = FINISHED;
      break;

    case FINISHED:
      // Nothing to do here
      do
      {

        digitalWrite( LRA_SEL,          LOW );          // SelectLRA
        digitalWrite( ERM_SEL,          HIGH );         // ERM disconnected
        
        drv2605.playFullHaptic( 6, 70, motorInfo[1].rated_duty, motorInfo[1].max_duty, compensation, backEMF, feedback );
        delay( 20 );
        
        digitalWrite( LRA_SEL,          HIGH );          // SelectLRA
        digitalWrite( ERM_SEL,          LOW );         // ERM disconnected   
        
        drv2605.playFullHaptic( 1, 82, motorInfo[0].rated_duty, motorInfo[0].max_duty, compensation1, backEMF1, feedback1 );
        
        delay( 20 );
      } while ( true );
      break;
  }
}

void setupTouchPad()
{
  //make sure the interrupt pin is an input and pulled high
  pinMode( IRQ_PIN, INPUT );
  digitalWrite( IRQ_PIN, HIGH );

  //output on ADC4 (PC4, SDA)
  DDRC |= 0b00010000;
  // Pull-ups on I2C Bus
  PORTC = 0b00110000;
  // initalize I2C bus. Wiring lib not used.
  i2cInit( 200 );

  timer.reset();
  while ( 0 == timer.check() );
  while ( 0 == timer.check() ); // wait at least 1/10 sec.

  // initialize mpr121
  mpr121QuickConfig();

  // Create and interrupt to trigger when a button
  // is hit, the IRQ pin goes low, and the function getKey is run.
  attachInterrupt( 0, keyPressed, FALLING );
} // setupTouchPad


void keyPressed() // this is interrupt INT0 !
{

  static char old_slide_pos;
  char  slide_pos = 0, zeroes = 0;

  touchstatus = mpr121Read( 0x01 ) << 8;
  touchstatus |= mpr121Read( 0x00 );

  if ( touchstatus & OVCF )
  {
    // Serial.println( F("OVER VOLT. !!!" ));
    mpr121Write( 0x80, 0x63 ); // Perform soft reset
    mpr121Write( 0x01, 0x80 ); // clear overcurrent flag
    mpr121Write( 0x80, 0x63 ); // Perform soft reset
    mpr121Write( 0x01, 0x80 ); // clear overcurrent flag
    mpr121Write( 0x80, 0x63 ); // Perform soft reset
    mpr121Write( 0x01, 0x80 ); // clear overcurrent flag
    mpr121QuickConfig();       // re-init mpr121
  }

  if (touchstatus) {                 //if one or more keys is pressed and no one was pressed

    if (!keylock) {

      keylock = touchstatus;
       keys_state_change = 1;

    }else{ //if key wasn't released i.ex. - slider
   
      touchstatus &= 0x01FF;  //clear BACK, PLAY, PMD_BUTTON
      if(touchstatus)  keys_state_change = 1;
      keylock = touchstatus;
    }
    
  } else if (!touchstatus ) { // if all key are released

    keylock = 0;

  }

} // keyPressed

//---------------------------------

void keys_service()
{
  volatile uint16_t touchstatus;
  static volatile uint16_t i=0, vibration=0;

  if ( keys_state_change )
  {

    if ( touchstatus != old_touchstatus )
    {

      switch ( state )
      {
        case WAITING_BACK : 
                    if (keylock == BACK){
                        i++;
                        state = nextStates[ i ];
                        vibration=1;
                    }
        break;
        case WAITING_PLAY : 
                    if (keylock == PLAY){
                      i++;
                      state = nextStates[ i ]; 
                      vibration =1  ;
                    }  
        break;
        case WAITING_PMD  : 
                    if (keylock == PMD_BUTTON){
                      i++;
                      state = nextStates[ i ];
                      vibration=1;
                      slider_values =0x01FF; //all bits for ELE0 to ELE8 are set
                    }
        break;
        case WAITING_SLIDE  : 
              
            if ((keylock != BACK ) && (keylock != PLAY )&&(keylock != PMD_BUTTON )){
              vibration=1 ; 
              slider_values &=~keylock;
              
                 if(!slider_values){
                  i++;  //all bits for ELE0 to ELE8 are set
                  state = nextStates[ i ];
                }
            }
        break;
      }

      if(vibration){
        drv2605.playFullHaptic( 6, keyPressEffects[ i ], motorInfo[1].rated_duty, motorInfo[1].max_duty, compensation, backEMF, feedback );
        vibration = 0;
      }
    
      touchstatus = old_touchstatus;
 
    }

    keys_state_change = 0;

  }
} // keyPressed

//----------------------------------------------
int freeRAM()
{
  extern int __heap_start, *__brkval;
  int v;
  return (int) &v - (__brkval == 0 ? (int) &__heap_start : (int) __brkval);
} // freeRAM
